////////////////////////////////////////////////////////////////////////
// SWIG wrapper for the cellular automata machine engine
// Don Hopkins
//
// Note: Delineate all Python-specific parts with #ifdef SWIGPYTHON/#endif.
// The same goes for all language-specific code: make it modular and 
// easy to support any language. 


////////////////////////////////////////////////////////////////////////
// ca module.


%module cam


////////////////////////////////////////////////////////////////////////
// Headers and globals to include in wrapper. 


#ifdef SWIGPYTHON


%{


// SWIG header code from cam.i.


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <cairo.h>
#include <pycairo.h>
//#include <pygobject.h>
#include <math.h>

#include "linuxcompat.h"
#include "cell.h"
#include "glue.h"

Pycairo_CAPI_t *Pycairo_CAPI;


%}


#endif


////////////////////////////////////////////////////////////////////////
// Initialization code for module init function. 


#ifdef SWIGPYTHON


%init %{

    // SWIG init code from cam.i.

    // Import the pycairo module's functions and types into this module. 
    // This enables us to convert between cairo_t and Python cairo.Context objects.
    Pycairo_IMPORT;

%}


#endif


////////////////////////////////////////////////////////////////////////
// Include SWIG utilities and typemaps.


#ifdef SWIGPYTHON


// This defines some Python specific in and out typemaps to convert 
// back and forth between Python "pycairo.Context" objects and 
// C "cairo_t *" pointers. 
%include "pycairo.i"


#endif


////////////////////////////////////////////////////////////////////////
// Include interface definitions. 


////////////////////////////////////////////////////////////////////////
// Include utilities


%include "cpointer.i"
%include "typemaps.i"


////////////////////////////////////////////////////////////////////////
// Type Maps


#ifdef SWIGPYTHON

// This may be built into swig now. If so, then use that and retire this. 

%typemap(in) std::string &(std::string tmp) {
	tmp = PyString_AsString($input);
	$1 = &tmp;
}


%typemap(out) std::string & {
	$result = Py_BuildValue("s", $1->c_str());
}

#endif


////////////////////////////////////////////////////////////////////////
// Interface definitions


%include "linuxcompat.h"
%include "cell.h"
%include "glue.h"


////////////////////////////////////////////////////////////////////////

