////////////////////////////////////////////////////////////////////////
// Python specific SWIG typemaps for pycairo.
// Don Hopkins, Internet Archive


#ifdef SWIGPYTHON


%typemap(in) cairo_t * {

    // SWIG cairo_t * in typemap from pycairo.i

    // Check to make sure PyObject "$input" is a cairo.Context object.
    if (!PyObject_TypeCheck(
	    $input,
	    &PycairoContext_Type)) {
        SWIG_exception_fail(
	    SWIG_TypeError,
	    "expected cairo.Context"); 
    }

    // Convert the pycairo cairo.Context wrapper "$input" to a cairo_t "$1". 
    $1 =
	PycairoContext_GET(
            $input);

}

%typemap(out) cairo_t * {

    // SWIG cairo_t * out typemap from pycairo.i

    // Convert the cairo_t "$1" to a cairo.Context wrapper "$result".
    $result = 
	PycairoContext_FromContext(
	    $1,
	    NULL, // type: NULL defaults to cairo.Context
	    NULL); // base: NULL means no base object

}


%typemap(in) cairo_surface_t * {

    // SWIG cairo_surface_t * in typemap from pycairo.i

    // Check to make sure PyObject "$input" is a cairo.Context object.
    if (!PyObject_TypeCheck(
	    $input,
	    &PycairoSurface_Type)) {
        SWIG_exception_fail(
	    SWIG_TypeError,
	    "expected cairo.Surface"); 
    }

    // Convert the pycairo cairo.Surface wrapper "$input" to a cairo_surface_t "$1". 
    $1 =
	PycairoSurface_GET(
            $input);

}

%typemap(out) cairo_surface_t * {

    // SWIG cairo_surface_t * out typemap from pycairo.i

    // Convert the cairo_surface_t "$1" to a cairo.Surface wrapper "$result".
    $result = 
	PycairoSurface_FromSurface(
	    $1,
	    NULL, // type: NULL defaults to cairo.Surface
	    NULL); // base: NULL means no base object

}


#endif


////////////////////////////////////////////////////////////////////////
