########################################################################
# Cellular Automata Machine Engine Utilities, by Don Hopkins.


########################################################################
# Imports

import sys, os, thread, time
import cam


########################################################################


class Engine:

    def __init__(self, w, h):
        self.width = w
        self.height = h
        self.rowBytes = (w + 3) & ~3
        self.colBytes = 1
        self.buf = cam.MakeBuffer(self.rowBytes, self.height)
        self.pal = cam.MakePalette()
        self.typeCode = 'B' # 1 byte unsigned integer
        self.cam = cam.MakeCam()
        self.cam.SetBuffer(self.width, self.height, self.buf, self.rowBytes)
        self.cam.SetSize(self.width, self.height)
        self.cam.SetWrap(3)
        self.cam.SetSteps(1)
        self.cam.SetFrob(-1)

    def __del__(self):
        if self.cam != None:
            cam.DestroyCam(self.cam)
            self.cam = None

    def Tick(self, steps=1, frob=-1):
        self.cam.SetSteps(steps)
        self.cam.SetFrob(frob)
        self.cam.DoRule()

    def GetCam(self):
        return self.cam

    def GetBuffer(self):
        return self.buf

    def GetPalette(self):
        return self.pal


########################################################################
